/*
** Module   : ToggleBox.java
** Abstract : toggle box implementation
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20140610 Initial implementation.
*/

package com.goldencode.p2j.ui.client;

import java.util.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.event.listener.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * An implementation of a "toggle box" with a text label.
 */
public abstract class ToggleBox
extends AbstractButton
implements DataContainer
{
   /** Toggle Box attribute container. */
   private ToggleBoxConfig config = null;

   /**
    * This constructor creates a toggle-box with a given text label.
    * 
    * @param   owner
    *          The parent container, frame or dialog which will include this toggle-box.
    * @param   text
    *          The string to be used as the toggle-box text.
    */
   public ToggleBox(Container owner, String text)
   {
      setTextLabel(text);
      
      config = new ToggleBoxConfig(ComponentConfig.TOGGLE_BOX, this);
      
      config.setName(text);
      config.setWidthCharsDirect(width());
      config.setHelp(null);
      addFocusListener(this);
   }

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   cfg
    *          Configuration data.
    */
   public ToggleBox(ComponentConfig cfg)
   {
      setTextLabel("");
      config = new ToggleBoxConfig(this, cfg);
      config.applyConfig(cfg);

      if (cfg.getLabel() != null)
         setText(cfg.getLabel());
      addFocusListener(this);
   }

   /**
    * Provide access to the attributes stored in the widget.
    * 
    * @return  Reference to attribute container.
    */
   @Override
   @SuppressWarnings("unchecked")
   public ToggleBoxConfig config()
   {
      return config;
   }

   // DataContainer methods implementations start
   /**
    * Get label text.
    * 
    * @return  label text.
    */
   @Override
   public String getText()
   {
      return textLabel();
   }

   /**
    * Getting the BaseDataType object representing the current value.
    *
    * @return  The value representing the current widget value.
    */
   @Override
   public BaseDataType getValue()
   {
      return new logical(config.isChecked());
   }

   /**
    * Getting the modification state of the widget.
    * 
    * @return  <code>true</code> if the widget value was modified,
    *          <code>false</code> otherwise.
    */
   @Override
   public boolean isModified()
   {
      return config.isModified();
   }

   /**
    * Sets the value of the toggle-box (the checked state) based on a
    * BaseDataType value.
    * 
    * @param    value
    *           The BaseDataType value of the logical value for toggle-box.
    */
   @Override
   public void setValue(BaseDataType value)
   {
      if (value instanceof logical)
      {
         config.setChecked(((logical)value).booleanValue());
      }
   }
   // DataContainer methods implementations end
   
   /**
    * Calculate the width of this widget.
    *
    * @return    The width in characters.
    */
   @Override
   public int width()
   {
      return config.getWidthChars() > 0 ? config.getWidthChars() : textLabel().length() + 3;
   }
   
   /**
    * Calculate the height of this widget.
    *
    * @return    The height in rows.
    */
   @Override
   public int height()
   {
      return config.getHeightChars() > 0 ? config.getHeightChars() : 1;
   }
   
   /**
    * Processes key events occurring on this object by dispatching them
    * to any registered KeyListener objects.
    *
    * @param    keyEvent
    *           The event to be handled.
    */
   @Override
   public void processKeyEvent(KeyInput keyEvent)
   {
      super.processKeyEventCommon(keyEvent);
   }

   /**
    * Set the new text for the component.
    * 
    * @param   text
    *          New text value.
    */
   @Override
   public void setText(String text)
   {
      setTextLabel(text);

      repaint();
      
      if (config.isAutoResize())
         config.setWidthCharsDirect(textLabel().length() + 3);

   }

   /**
    * Process basic events - TAB, BACK-TAB and ENTER key event.
    * 
    * @param   event
    *          Key event to process.
    */
   @Override
   protected void handleBasicEvents(KeyInput event)
   {
      super.processKeyEvent(event);

      if (event.isConsumed())
      {
         return;
      }

      int key = event.keyCode();
      if (key == '\t')
      {
         parent().nextFocus();
         return;
      }
      else if (key == Key.VK_BACK_TAB)
      {
         parent().prevFocus();
         return;
      }

      if (super.isEnabled() && (key == Key.VK_ENTER || key == ' '))
      {
         // inverse the CHECKED state
         config.setChecked(!config.isChecked());
         config.setModified(true);
         refresh();
         // Generate VALUE_CHANGED event
         KeyInput valueChangedEvent = new KeyInput(this, EventType.KEY_TYPED,
                                                   Keyboard.SE_VALUE_CHANGED);
         EventManager.postEvent(valueChangedEvent);
         // eat the original event
         event.consume();
      }
   }
}
