/*
** Module   : Widget.java
** Abstract : Common interface for all widgets. 
**
** Copyright (c) 2010-14, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100629          Created initial version
** 002 SIY 20100804          Changes in imports, some refactorings.
** 003 SIY 20100817          Fixed return type for ancestor() method.
** 004 SIY 20100822          Added config() and isFocusable() methods.
** 005 SIY 20101122          Switching from KeyEvent to KeyInput.
** 006 SIY 20111115          Added getter/setter for physical widget location.
** 007 EVL 20140617          Added method to check if the widget needs the cursor to be on.
*/

package com.goldencode.p2j.ui.client.widget;

import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.event.listener.*;

/**
 * Common interface implemented by all widgets.
 */
public interface Widget
{
   /**
    * Get access to {@link OutputManager} instance.
    * 
    * @return  reference to {@link OutputManager}.
    */
   public OutputManager screen();

   /**
    * Add focus listener.
    * 
    * @param   listener
    *          Listener to add.
    */
   public void addFocusListener(FocusListener listener);

   /**
    * Add key listener.
    * 
    * @param   listener
    *          Listener to add.
    */
   public void addKeyListener(KeyListener listener);

   /**
    * Get ancestor window for this widget.
    * 
    * @return  ancestor window.
    */
   public Widget ancestor();

   /**
    * Get widget bounding rectangle.
    * 
    * @return  bounding rectangle.
    */
   public Rectangle bounds();

   /**
    * Get widget dimension.
    * 
    * @return  widget dimension.
    */
   public Dimension dimension();

   /**
    * Check if widget is displayed.
    * 
    * @return  <code>true</code> if widget is displayed.
    */
   public boolean isDisplayed();

   /**
    * Draw widget.
    */
   public void draw();

   /**
    * Draw caret.
    */
   public void drawCaret();

   /**
    * Check if widget is enabled.
    * 
    * @return  <code>true</code> if widget is enabled.
    */
   public boolean isEnabled();

   /**
    * Check if widget is focus traversable.
    * 
    * @return  <code>true</code> if widget is focus traversable.
    */
   public boolean focusTraversable();

   /**
    * Force widget visibility change without associated repainting.
    * 
    * @param   visibility
    *          New visibility state.
    */
   public void forceVisibility(boolean visibility);

   /**
    * Check if widget has focus.
    * 
    * @return  <code>true</code> if widget has focus.
    */
   public boolean hasFocus();

   /**
    * Get widget height.
    * 
    * @return  widget height.
    */
   public int height();

   /**
    * Get current hidden state.
    * 
    * @return   hidden state.
    */
   public boolean hidden();

   /**
    * Hide widget.
    */
   public void hide();

   /**
    * Get current highlighted state.
    * 
    * @return   highlighted state.
    */
   public boolean highlighted();

   /**
    * Get widget location relative to container.
    * 
    * @return  relative widget location.
    */
   public Point location();

   /**
    * Get widget minimum size.
    * 
    * @return  widget minimum size.
    */
   public Dimension minimumSize();

   /**
    * Get widget name.
    * 
    * @return  widget name.
    */
   public String name();

   /**
    * Get widget parent.
    * 
    * @return  widget parent.
    */
   public Container parent();

   /**
    * Process focus event.
    * 
    * @param   event
    *          Event to process.
    */
   public void processFocusEvent(FocusEvent event);

   /**
    * Process key event.
    * 
    * @param   event
    *          Key event to process.
    */
   public void processKeyEvent(KeyInput event);

   /**
    * Process key listeners.
    * 
    * @param    event
    *           Keyboard event.
    */
   public void processKeyListeners(KeyInput event);

   /**
    * Request widget repainting.
    */
   public void repaint();

   /**
    * Request repainting of the area occupied by widget.
    * 
    * Only the given update rectangle will be repainted (note that it is
    * clipped by all parent containers). No event is posted if the widget
    * itself or any of the parent containers are hidden.
    * 
    * @param    ur
    *           The update rectangle (relative to the origin of its parent
    *           Container) which will be repainted. Note that it is clipped by
    *           all parent containers.
    */
   public void repaint(Rectangle ur);

   /**
    * Request focus.
    */
   public void requestFocus();

   /**
    * Request screen synchronization.  
    */
   public void requestSync();

   /**
    * Get absolute location of widget in screen coordinates.
    * 
    * @return  absolute widget location.
    */
   public Point screenLocation();

   /**
    * Enable/disable widget.
    * 
    * @param   enabled
    */
   public void setEnabled(boolean enabled);

   /**
    * Tracks the hidden state of the widget.
    * 
    * @param    hidden
    *           New hidden state.
    */
   public void setHidden(boolean hidden);

   /**
    * Tracks the highlighted state of the widget.
    * 
    * @param    highlighted
    *           New highlighted state.
    */
   public void setHighlighted(boolean highlighted);

   /**
    * Set widget relative location.
    * 
    * @param   x
    *          Widget X.
    * @param   y
    *          Widget Y.
    */
   public void setLocation(int x, int y);

   /**
    * Set widget relative location.
    * 
    * @param   origin
    *          Location to set.
    */
   public void setLocation(Point origin);

   /**
    * Set widget name.
    * 
    * @param   name
    *          New widget name.
    */
   public void setName(String name);

   /**
    * Set widget parent.
    * 
    * @param   container
    *          Parent to set.
    */
   public void setParent(Container container);

   /**
    * Hide/show widget.
    * 
    * @param   visible
    *          New visible state.
    */
   public void setVisible(boolean visible);

   /**
    * Show widget.
    */
   public void show();

   /**
    * Check if widget is totally obscured.
    * 
    * @return  <code>true</code> if widget is totally obscured.
    */
   public boolean isTotallyObscured();

   /**
    * Check if widget is visible.
    * 
    * @return  <code>true</code> if widget is visible.
    */
   public boolean isVisible();

   /**
    * Check if widget needs in cursor to be on when having the input focus.
    * 
    * @return  <code>true</code> if cursor is required.
    */
   public boolean forceCursorOnFocus();

   /**
    * Get widget width.
    * 
    * @return  widget width.
    */
   public int width();

   /**
    * Process event and dispatch it to specific method.
    * 
    * @param   evt
    *          Event to process.
    */
   public void processEvent(Event evt);
   
   /**
    * Get externalizable widget config which can be used to restore widget 
    * state after via network. 
    * Note that only very limited set of widgets actually have config.
    * 
    * @param   <T>
    *          Config type.
    *          
    * @return  Reference to widget config.
    */
   public<T extends WidgetConfig> T config();

   /**
    * Checks whether this widget can receive focus generally.
    * 
    * @return  <code>true</code> if this widget can be focused under
    *          certain conditions. <code>false</code> if, and only if, this
    *          widget can never receive focus.    
    */
   boolean isFocusable();

   /**
    * Set widget relative physical location.
    * 
    * @param   x
    *          Widget physical X.
    * @param   y
    *          Widget physical Y.
    */
   void setPhysicalLocation(int x, int y);

   /**
    * Get widget physical location relative to container.
    * 
    * @return  relative widget location.
    */
   public Point physicalLocation();
}